@@ -2,7 +2,7 @@
 #define _PRIVATE_SLANG_H_
 /* header file for S-Lang internal structures that users do not (should not)
    need.  Use slang.h for that purpose. */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -66,6 +66,7 @@
 #define _SLANG_BC_CALL_DIRECT		0x24
 #define _SLANG_BC_CALL_DIRECT_FRAME	0x25
 #define _SLANG_BC_UNARY			0x26
+#define _SLANG_BC_UNARY_FUNC		0x27
 
 #define _SLANG_BC_DEREF_ASSIGN		0x30
 #define _SLANG_BC_SET_LOCAL_LVALUE	0x31
@@ -269,16 +270,6 @@ extern int _SLpush_slang_obj (SLang_Object_Type *);
 extern char *_SLexpand_escaped_char(char *, char *);
 extern void _SLexpand_escaped_string (char *, char *, char *);
 
-extern int _SLreverse_stack (int);
-extern int _SLroll_stack (int);
-/* If argument *p is positive, the top |*p| objects on the stack are rolled
- * up.  If negative, the stack is rolled down.
- */
-extern int _SLdup_n (int n);
-
-extern SLang_Array_Type *_SLang_apropos (char *, char *, unsigned int);
-extern void _SLang_implements_intrinsic (char *);
-
 /* returns a pointer to an SLstring string-- use SLang_free_slstring */
 extern char *_SLstringize_object (SLang_Object_Type *);
 extern int _SLdump_objects (char *, SLang_Object_Type *, unsigned int, int);
@@ -286,19 +277,22 @@ extern int _SLdump_objects (char *, SLang_Object_Type *, unsigned int, int);
 extern SLang_Object_Type *_SLRun_Stack;
 extern SLang_Object_Type *_SLStack_Pointer;
 
-typedef struct _SLNameSpace_List_Type
+struct _SLang_NameSpace_Type
 {
-   struct _SLNameSpace_List_Type *next;
+   struct _SLang_NameSpace_Type *next;
    char *name;			       /* this is the load_type name */
    char *namespace_name;	       /* this name is assigned by implements */
    unsigned int table_size;
    SLang_Name_Type **table;
-}
-_SLNameSpace_List_Type;
-extern _SLNameSpace_List_Type *_SLns_allocate_namespace (char *, unsigned int);
-extern _SLNameSpace_List_Type *_SLns_find_namespace (char *);
-extern int _SLns_set_namespace_name (_SLNameSpace_List_Type *, char *);
-extern SLang_Array_Type *_SLnspace_apropos (_SLNameSpace_List_Type *, char *, unsigned int);
+};
+extern SLang_NameSpace_Type *_SLns_allocate_namespace (char *, unsigned int);
+extern SLang_NameSpace_Type *_SLns_find_namespace (char *);
+extern int _SLns_set_namespace_name (SLang_NameSpace_Type *, char *);
+extern SLang_Array_Type *_SLnspace_apropos (SLang_NameSpace_Type *, char *, unsigned int);
+extern void _SLang_use_namespace_intrinsic (char *name);
+extern char *_SLang_cur_namespace_intrinsic (void);
+extern SLang_Array_Type *_SLang_apropos (char *, char *, unsigned int);
+extern void _SLang_implements_intrinsic (char *);
 
 extern int _SLang_Trace;
 extern int _SLstack_depth(void);
@@ -324,6 +318,7 @@ extern void _SLpack (void);
 extern void _SLunpack (char *, SLang_BString_Type *);
 extern void _SLpack_pad_format (char *);
 extern unsigned int _SLpack_compute_size (char *);
+extern int _SLusleep (unsigned long);
 
 /* frees upon error.  NULL __NOT__ ok. */
 extern int _SLang_push_slstring (char *);
@@ -347,9 +342,11 @@ extern unsigned int _SLpc_convert_scancode (unsigned int, unsigned int, int);
 #define _SLTT_KEY_ALT		4
 #endif
 
-#ifdef REAL_UNIX_SYSTEM
-extern int SLtt_tigetflag (char *, char **);
-#endif
+typedef struct _SLterminfo_Type SLterminfo_Type;
+extern SLterminfo_Type *_SLtt_tigetent (char *);
+extern char *_SLtt_tigetstr (SLterminfo_Type *, char *);
+extern int _SLtt_tigetnum (SLterminfo_Type *, char *);
+extern int _SLtt_tigetflag (SLterminfo_Type *, char *);
 
 #if SLTT_HAS_NON_BCE_SUPPORT
 extern int _SLtt_get_bce_color_offset (void);
