@@ -1,14 +1,14 @@
 #ifndef DAVIS_SLANG_H_
 #define DAVIS_SLANG_H_
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#define SLANG_VERSION 10310	       /* 1.3.10 */
-#define SLANG_VERSION_STRING "B1.3.10"
+#define SLANG_VERSION 10404
+#define SLANG_VERSION_STRING "1.4.4"
 
 /*{{{ System Dependent Macros and Typedefs */
 
@@ -259,6 +259,19 @@ extern int SLadd_iconstant_table (SLang_IConstant_Type *, char *);
 extern int SLadd_dconstant_table (SLang_DConstant_Type *, char *);
 extern int SLadd_istruct_table (SLang_IStruct_Field_Type *, VOID_STAR, char *);
 
+typedef struct _SLang_NameSpace_Type SLang_NameSpace_Type;
+
+extern int SLns_add_intrin_fun_table (SLang_NameSpace_Type *, SLang_Intrin_Fun_Type *, char *);
+extern int SLns_add_intrin_var_table (SLang_NameSpace_Type *, SLang_Intrin_Var_Type *, char *);
+extern int SLns_add_app_unary_table (SLang_NameSpace_Type *, SLang_App_Unary_Type *, char *);
+extern int SLns_add_math_unary_table (SLang_NameSpace_Type *, SLang_Math_Unary_Type *, char *);
+extern int SLns_add_iconstant_table (SLang_NameSpace_Type *, SLang_IConstant_Type *, char *);
+extern int SLns_add_dconstant_table (SLang_NameSpace_Type *, SLang_DConstant_Type *, char *);
+extern int SLns_add_istruct_table (SLang_NameSpace_Type *, SLang_IStruct_Field_Type *, VOID_STAR, char *);
+
+extern SLang_NameSpace_Type *SLns_create_namespace (char *);
+extern void SLns_delete_namespace (SLang_NameSpace_Type *);
+
 typedef struct SLang_Load_Type
 {
    int type;
@@ -651,6 +664,7 @@ extern int SLang_set_array_element (SLang_Array_Type *, int *, VOID_STAR);
 #define SL_NOT_IMPLEMENTED		9
 #define SL_MALLOC_ERROR			10
 #define SL_OVERFLOW			11
+#define SL_FLOATING_EXCEPTION		12
 
 /* Compatibility */
 #define USER_BREAK SL_USER_BREAK
@@ -664,6 +678,7 @@ extern int SLang_set_array_element (SLang_Array_Type *, int *, VOID_STAR);
   /* Prompt to use when reading from stdin */
   extern int SLang_Version;
   extern char *SLang_Version_String;
+extern char *SLang_Doc_Dir;
 
 extern void (*SLang_VMessage_Hook) (char *, va_list);
 extern void SLang_vmessage (char *, ...);
@@ -828,6 +843,7 @@ extern void SLang_free_ref (SLang_Ref_Type *);
 extern int SLang_assign_to_ref (SLang_Ref_Type *, unsigned char, VOID_STAR);
 extern SLang_Name_Type *SLang_pop_function (void);
 extern SLang_Name_Type *SLang_get_fun_from_ref (SLang_Ref_Type *);
+extern void SLang_free_function (SLang_Name_Type *f);
 
    extern int SLang_is_defined(char *);
    /* Return non-zero is p1 is defined otherwise returns 0. */
@@ -870,6 +886,14 @@ extern void SLexpand_escaped_string (register char *, register char *,
 extern SLang_Name_Type *SLang_get_function (char *);
 extern void SLang_release_function (SLang_Name_Type *);
 
+extern int SLreverse_stack (int);
+extern int SLroll_stack (int);
+/* If argument p is positive, the top p objects on the stack are rolled
+ * up.  If negative, the stack is rolled down.
+ */
+extern int SLdup_n (int n);
+/* Duplicate top n elements of stack */
+
 extern int SLang_peek_at_stack1 (void);
 extern int SLang_peek_at_stack (void);
 /* Returns type of next object on stack-- -1 upon stack underflow. */
@@ -949,6 +973,8 @@ extern SLFile_FD_Type *SLfile_create_fd (char *, int);
 extern void SLfile_free_fd (SLFile_FD_Type *);
 extern int SLfile_push_fd (SLFile_FD_Type *);
 extern int SLfile_pop_fd (SLFile_FD_Type **);
+extern int SLfile_get_fd (SLFile_FD_Type *, int *);
+extern SLFile_FD_Type *SLfile_dup_fd (SLFile_FD_Type *f0);
 extern int SLang_init_posix_io (void);
 
 typedef double (*SLang_To_Double_Fun_Type)(VOID_STAR);
@@ -1189,6 +1215,7 @@ extern int SLtt_Use_Ansi_Colors;
 extern int SLtt_Ignore_Beep;
 #if defined(REAL_UNIX_SYSTEM)
 extern int SLtt_Force_Keypad_Init;
+extern int SLang_TT_Write_FD;
 #endif
 
 #ifndef IBMPC_SYSTEM
@@ -1255,12 +1282,19 @@ extern void SLtt_set_alt_char_set (int);
 extern int SLtt_write_to_status_line (char *, int);
 extern void SLtt_disable_status_line (void);
 # ifdef REAL_UNIX_SYSTEM
-   extern char *SLtt_tgetstr (char *);
-   extern int SLtt_tgetnum (char *);
-   extern int SLtt_tgetflag (char *);
-   extern char *SLtt_tigetent (char *);
-   extern char *SLtt_tigetstr (char *, char **);
-   extern int SLtt_tigetnum (char *, char **);
+/* These are termcap/terminfo routines that assume SLtt_initialize has
+ * been called.
+ */
+extern char *SLtt_tgetstr (char *);
+extern int SLtt_tgetnum (char *);
+extern int SLtt_tgetflag (char *);
+
+/* The following are terminfo-only routines -- these prototypes will change
+ * in V2.x.
+ */
+extern char *SLtt_tigetent (char *);
+extern char *SLtt_tigetstr (char *, char **);
+extern int SLtt_tigetnum (char *, char **);
 # endif
 #endif
 
@@ -1561,6 +1595,7 @@ extern int SLsig_unblock_signals (void);
 extern int SLsystem (char *);
 
 extern char *SLerrno_strerror (int);
+extern int SLerrno_set_errno (int);
 
 /*}}}*/
 
@@ -1661,6 +1696,9 @@ extern char *SLang_Error_Message;
 int SLadd_intrinsic_variable (char *, VOID_STAR, unsigned char, int);
 int SLadd_intrinsic_function (char *, FVOID_STAR, unsigned char, unsigned int,...);
 
+int SLns_add_intrinsic_variable (SLang_NameSpace_Type *, char *, VOID_STAR, unsigned char, int);
+int SLns_add_intrinsic_function (SLang_NameSpace_Type *, char *, FVOID_STAR, unsigned char, unsigned int,...);
+
 extern void SLadd_at_handler (long *, char *);
 
 #define MAKE_INTRINSIC_N(n,f,out,in,a1,a2,a3,a4,a5,a6,a7) \
@@ -1744,6 +1782,9 @@ extern void SLadd_at_handler (long *, char *);
 #define SLANG_END_MATH_UNARY_TABLE MAKE_MATH_UNARY(NULL,0)
 #define SLANG_END_INTRIN_VAR_TABLE MAKE_VARIABLE(NULL,NULL,0,0)
 #define SLANG_END_ICONST_TABLE MAKE_ICONSTANT(NULL,0)
+#define SLANG_END_ISTRUCT_TABLE {NULL, 0, 0, 0}
+
+   
 
 /*}}}*/
 
@@ -1869,11 +1910,16 @@ extern char *SLpath_dircat (char *, char *);
 extern char *SLpath_find_file_in_path (char *, char *);
 extern char *SLpath_dirname (char *);
 extern int SLpath_file_exists (char *);
+extern char *SLpath_pathname_sans_extname (char *);
 
 /*}}}*/
 
 extern int SLang_set_module_load_path (char *);
 
+#define SLANG_MODULE(name) \
+   extern int init_##name##_module_ns (char *); \
+   extern void deinit_##name##_module (void)
+
 #if 0
 {
 #endif
