@@ -1,5 +1,5 @@
 
-/* Copyright (c) 1998, 1999 John E. Davis
+/* Copyright (c) 1998, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -10,6 +10,10 @@
 
 #include <math.h>
 
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
 #include "slang.h"
 #include "_slang.h"
 
@@ -1111,15 +1115,18 @@ int SLang_pop_double(double *x, int *convertp, int *ip)
 	convert = 1;
 	break;
 
+      case SLANG_CHAR_TYPE: *x = (double) obj.v.char_val; break;
+      case SLANG_UCHAR_TYPE: *x = (double) obj.v.uchar_val; break;
+      case SLANG_SHORT_TYPE: *x = (double) obj.v.short_val; break;
+      case SLANG_USHORT_TYPE: *x = (double) obj.v.ushort_val; break;
+      case SLANG_UINT_TYPE: *x = (double) obj.v.uint_val; break;
+      case SLANG_LONG_TYPE: *x = (double) obj.v.long_val; break;
+      case SLANG_ULONG_TYPE: *x = (double) obj.v.ulong_val; break;
+	
       default:
-	if (0 == IS_INTEGER_TYPE (obj.data_type))
-	  {
-	     _SLclass_type_mismatch_error (SLANG_DOUBLE_TYPE, obj.data_type);
-	     SLang_free_object (&obj);
-	     return -1;
-	  }
-	/* All integers are store on stack as a long */
-	*x = (double) obj.v.long_val;
+	_SLclass_type_mismatch_error (SLANG_DOUBLE_TYPE, obj.data_type);
+	SLang_free_object (&obj);
+	return -1;
      }
 
    if (convertp != NULL) *convertp = convert;
@@ -1431,6 +1438,11 @@ int _SLarith_register_types (void)
    int a_type, b_type;
    int i, j;
 
+#if defined(HAVE_SETLOCALE) && defined(LC_NUMERIC)
+   /* make sure decimal point it used --- the parser requires it */
+   (void) setlocale (LC_NUMERIC, "C"); 
+#endif
+
    for (i = 0; i < 8; i++)
      {
 	Integer_Info_Type *info;
