@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -95,11 +95,12 @@ void (*_SLtt_color_changed_hook)(void);
 
 #if SLTT_HAS_NON_BCE_SUPPORT
 static int Bce_Color_Offset = 0;
-static int Can_Background_Color_Erase;
 #endif
+static int Can_Background_Color_Erase = 1;
 
 /* -1 means unknown */
 int SLtt_Has_Status_Line = -1;	       /* hs */
+int SLang_TT_Write_FD = -1;
 
 static int Automatic_Margins;
 /* static int No_Move_In_Standout; */
@@ -159,8 +160,8 @@ static Ansi_Color_Type Ansi_Color_Map[JMAX_COLORS] =
      {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL}
 };
 
-static char *Color_Fg_Str;
-static char *Color_Bg_Str;
+static char *Color_Fg_Str = "\033[3%dm";
+static char *Color_Bg_Str = "\033[4%dm";
 static char *Default_Color_Fg_Str = "\033[39m";
 static char *Default_Color_Bg_Str = "\033[49m";
 
@@ -190,7 +191,10 @@ static char *Curs_Up_Str;
 static char *Curs_F_Str;    /* RI termcap string */
 static char *Cursor_Visible_Str;    /* ve termcap string */
 static char *Cursor_Invisible_Str;    /* vi termcap string */
-
+#if 0
+static char *Start_Mouse_Rpt_Str;  /* Start mouse reporting mode */
+static char *End_Mouse_Rpt_Str;  /* End mouse reporting mode */
+#endif
 static char *Start_Alt_Chars_Str;  /* as */
 static char *End_Alt_Chars_Str;   /* ae */
 static char *Enable_Alt_Char_Set;  /* eA */
@@ -205,7 +209,7 @@ static char *Disable_Status_line_Str;  /* ds */
 static char *Return_From_Status_Line_Str;   /* fs */
 static char *Goto_Status_Line_Str;     /* ts */
 static int Num_Status_Line_Columns;    /* ws */
-static int Status_Line_Esc_Ok;	       /* es */
+/* static int Status_Line_Esc_Ok;	 */       /* es */
 
 /* static int Len_Curs_F_Str = 5; */
 
@@ -231,7 +235,7 @@ static unsigned char *Output_Bufferp = Output_Buffer;
 
 unsigned long SLtt_Num_Chars_Output;
 
-static int sl_usleep (unsigned long usecs)
+int _SLusleep (unsigned long usecs)
 {
 #if !defined(VMS) || (__VMS_VER >= 70000000)
    struct timeval tv;
@@ -254,21 +258,21 @@ int SLtt_flush_output (void)
    total = 0;
    while (n > 0)
      {
-	nwrite = write (fileno(stdout), (char *) Output_Buffer + total, n);
+	nwrite = write (SLang_TT_Write_FD, (char *) Output_Buffer + total, n);
 	if (nwrite == -1)
 	  {
 	     nwrite = 0;
 #ifdef EAGAIN
 	     if (errno == EAGAIN)
 	       {
-		  sl_usleep (100000);   /* 1/10 sec */
+		  _SLusleep (100000);   /* 1/10 sec */
 		  continue;
 	       }
 #endif
 #ifdef EWOULDBLOCK
 	     if (errno == EWOULDBLOCK)
 	       {
-		  sl_usleep (100000);
+		  _SLusleep (100000);
 		  continue;
 	       }
 #endif
@@ -868,7 +872,7 @@ void SLtt_beep (void)
 	  {
 	     tt_write_string ("\033[?5h");
 	     SLtt_flush_output ();
-	     sl_usleep (50000);
+	     _SLusleep (50000);
 	     tt_write_string ("\033[?5l");
 	  }
 #endif
@@ -876,10 +880,35 @@ void SLtt_beep (void)
    SLtt_flush_output ();
 }
 
+static void del_eol (void)
+{
+   int c;
+
+   if (Del_Eol_Str != NULL)
+     {
+	tt_write_string(Del_Eol_Str);
+	return;
+     }
+
+   c = Cursor_c;
+   /* Avoid writing to the lower right corner.  If the terminal does not
+    * have Del_Eol_Str, then it probably does not have what it takes to play
+    * games with insert for for a space into that corner.
+    */
+   if (Cursor_r + 1 < SLtt_Screen_Rows)
+     c++;
+
+   while (c < SLtt_Screen_Cols)
+     {
+	tt_write (" ", 1);
+	c++;
+     }
+}
+
 void SLtt_del_eol (void)
 {
    if (Current_Fgbg != 0xFFFFFFFFU) SLtt_normal_video ();
-   tt_write_string(Del_Eol_Str);
+   del_eol ();
 }
 
 typedef struct
@@ -1741,9 +1770,10 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
 			 { /* kanji match ? */
 			    if (!COLOR_EQS(*q, *p) || !COLOR_EQS(*(q+1), *(p+1)))
 			      {
-				 /* code is match ,but color is diff */
+				 /* code is match, but color is diff */
 				 *buf++ = *p++;
 				 *buf++ = *p++;
+				 q += 2;
 				 continue;
 			      }
 			    /* really match ! */
@@ -1872,7 +1902,7 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
    if (q < qmax) 
      {
 	SLtt_reverse_video (space_char >> 8);
-	tt_write_string (Del_Eol_Str);
+	del_eol ();
      }
    
 #if SLTT_USE_INSERT_HACK
@@ -1924,76 +1954,92 @@ static void get_color_info (void)
 
 #ifdef __unix__
 
-# ifndef USE_TERMCAP
-static char *Tbuf;
-static char *Tstr_Buf;
-
-#  define tgetstr SLtt_tigetstr
-#  define tgetent SLtt_tigetent
-#  define TGETNUM(x) SLtt_tigetnum((x), &Tbuf)
-#  define TGETFLAG(x) (SLtt_tigetflag((x), &Tbuf) > 0)
-# else
+static int Termcap_Initalized = 0;
 
+#ifdef USE_TERMCAP
+/* Termcap based system */
+static char Termcap_Buf[4096];
+static char Termcap_String_Buf[4096];
+static char *Termcap_String_Ptr;
 extern char *tgetstr(char *, char **);
 extern int tgetent(char *, char *);
 extern int tgetnum(char *);
 extern int tgetflag(char *);
-static char Tstr_Buf[1024];
-static char Tbuf[4096];
-#  define TGETNUM tgetnum
-#  define TGETFLAG tgetflag
-# endif
+#else
+/* Terminfo */
+static SLterminfo_Type *Terminfo;
+#endif
 
-static char *my_tgetstr(char *what, char **p)
+#define TGETFLAG(x) (SLtt_tgetflag(x) > 0)
+
+static char *fixup_tgetstr (char *what)
 {
    register char *w, *w1;
    char *wsave;
-   what = tgetstr(what, p);
-   if (what != NULL)
+   
+   if (what == NULL)
+     return NULL;
+
+   /* Check for AIX brain-damage */
+   if (*what == '@')
+     return NULL;
+
+   /* lose pad info --- with today's technology, term is a loser if
+    it is really needed */
+   while ((*what == '.') ||
+	  ((*what >= '0') && (*what <= '9'))) what++;
+   if (*what == '*') what++;
+   
+   /* lose terminfo padding--- looks like $<...> */
+   w = what;
+   while (*w) if ((*w++ == '$') && (*w == '<'))
      {
-	/* Check for AIX brain-damage */
-	if (*what == '@')
-	  return NULL;
-
-	/* lose pad info --- with today's technology, term is a loser if
-	   it is really needed */
-	while ((*what == '.') ||
-	       ((*what >= '0') && (*what <= '9'))) what++;
-	if (*what == '*') what++;
-
-	/* lose terminfo padding--- looks like $<...> */
-        w = what;
-	while (*w) if ((*w++ == '$') && (*w == '<'))
-	  {
-	     w1 = w - 1;
-	     while (*w && (*w != '>')) w++;
-	     if (*w == 0) break;
-	     w++;
-	     wsave = w1;
-	     while ((*w1++ = *w++) != 0);
-	     w = wsave;
-	  }
-	if (*what == 0) what = NULL;
+	w1 = w - 1;
+	while (*w && (*w != '>')) w++;
+	if (*w == 0) break;
+	w++;
+	wsave = w1;
+	while ((*w1++ = *w++) != 0);
+	w = wsave;
      }
-   return(what);
+
+   if (*what == 0) what = NULL;
+   return what;
 }
 
 char *SLtt_tgetstr (char *s)
 {
-# ifdef USE_TERMCAP
-   static
-# endif
-     char *p = Tstr_Buf;
-   return my_tgetstr (s, &p);
+   if (Termcap_Initalized == 0)
+     return NULL;
+   
+#ifdef USE_TERMCAP
+   s = tgetstr (s, &Termcap_String_Ptr);
+#else
+   s = _SLtt_tigetstr (Terminfo, s);
+#endif
+   return fixup_tgetstr (s);
 }
 
 int SLtt_tgetnum (char *s)
 {
-   return TGETNUM (s);
+   if (Termcap_Initalized == 0)
+     return -1;
+#ifdef USE_TERMCAP
+   return tgetnum (s);
+#else
+   return _SLtt_tigetnum (Terminfo, s);
+#endif
 }
+
 int SLtt_tgetflag (char *s)
 {
-   return TGETFLAG (s);
+   if (Termcap_Initalized == 0)
+     return -1;
+#ifdef USE_TERMCAP
+   return tgetflag (s);
+#else
+   return _SLtt_tigetflag (Terminfo, s);
+#endif
 }
 
 static int Vt100_Like = 0;
@@ -2036,6 +2082,12 @@ int SLtt_initialize (char *term)
    int is_xterm;
    int almost_vtxxx;
 
+   if (SLang_TT_Write_FD == -1)
+     {
+	/* Apparantly, this cannot fail according to the man pages. */
+	SLang_TT_Write_FD = fileno (stdout);
+     }
+   
    if (term == NULL)
      {
 	term = getenv ("TERM");
@@ -2054,14 +2106,17 @@ int SLtt_initialize (char *term)
    if (strcmp(t, "vt52") && (*t++ == 'v') && (*t++ == 't')
        && (ch = *t, (ch >= '1') && (ch <= '9'))) Vt100_Like = 1;
 
-   is_xterm = !strncmp (term, "xterm", 5);
+   is_xterm = ((0 == strncmp (term, "xterm", 5))
+	       || (0 == strncmp (term, "rxvt", 4))
+	       || (0 == strncmp (term, "Eterm", 5)));
+
    almost_vtxxx = (Vt100_Like
 		   || Linux_Console
 		   || is_xterm
 		   || !strcmp (term, "screen"));
 
 # ifndef USE_TERMCAP
-   if (NULL == (Tbuf = tgetent (term)))
+   if (NULL == (Terminfo = _SLtt_tigetent (term)))
      {
 	if (almost_vtxxx) /* Special cases. */
 	  {
@@ -2073,12 +2128,14 @@ int SLtt_initialize (char *term)
 	  }
 	return -1;
      }
-   Tstr_Buf = Tbuf;
 # else				       /* USE_TERMCAP */
-   if (1 != tgetent(Tbuf, term))
+   if (1 != tgetent(Termcap_Buf, term))
      return -1;
+   Termcap_String_Ptr = Termcap_String_Buf;
 # endif				       /* NOT USE_TERMCAP */
 
+   Termcap_Initalized = 1;
+
    Cls_Str = SLtt_tgetstr ("cl");
    Curs_Pos_Str = SLtt_tgetstr ("cm");
 
@@ -2200,8 +2257,8 @@ int SLtt_initialize (char *term)
 	Disable_Status_line_Str = SLtt_tgetstr ("ds");
 	Return_From_Status_Line_Str = SLtt_tgetstr ("fs");
 	Goto_Status_Line_Str = SLtt_tgetstr ("ts");
-	Status_Line_Esc_Ok = TGETFLAG("es");
-	Num_Status_Line_Columns = TGETNUM("ws");
+	/* Status_Line_Esc_Ok = TGETFLAG("es"); */
+	Num_Status_Line_Columns = SLtt_tgetnum ("ws");
 	if (Num_Status_Line_Columns < 0) Num_Status_Line_Columns = 0;
      }
 
@@ -2233,7 +2290,7 @@ int SLtt_initialize (char *term)
 
    if (Worthless_Highlight == 0)
      {				       /* Magic cookie glitch */
-	Worthless_Highlight = (TGETNUM ("sg") > 0);
+	Worthless_Highlight = (SLtt_tgetnum ("sg") > 0);
      }
 
    if (Worthless_Highlight)
@@ -2248,7 +2305,7 @@ int SLtt_initialize (char *term)
 	Color_Bg_Str = SLtt_tgetstr ("Sb");   /* setb */
      }
 
-   if (-1 == (Max_Terminfo_Colors = SLtt_tgetnum ("Co")))
+   if ((Max_Terminfo_Colors = SLtt_tgetnum ("Co")) < 0)
      Max_Terminfo_Colors = 8;
 
    if ((Color_Bg_Str != NULL) && (Color_Fg_Str != NULL))
@@ -2509,9 +2566,12 @@ void SLtt_get_screen_size (void)
 	if (s != NULL) c = atoi (s);
      }
 
+   if (r <= 0) r = 24;
+   if (c <= 0) c = 80;
+#if 0
    if ((r <= 0) || (r > 200)) r = 24;
    if ((c <= 0) || (c > 250)) c = 80;
-
+#endif
    SLtt_Screen_Rows = r;
    SLtt_Screen_Cols = c;
 }
