@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -7,21 +7,38 @@
 
 #include "slinclud.h"
 
+#ifdef __DJGPP__
+# define _NAIVE_DOS_REGS
+#endif
+
 #include <dos.h>
 
 #if defined (__EMX__)
 #  define int86		_int86
 #  define delay		_sleep2
 #endif	/* __EMX__ */
+
 #if defined (__WATCOMC__)
 #  include <conio.h>
 #  include <bios.h>
 #  define int86	int386
-#else
-#  define _NKEYBRD_READ		0x0
-#  define _NKEYBRD_READY	0x1
-#  define _NKEYBRD_SHIFTSTATUS	0x2
-#endif	/* __WATCOMC__ */
+#endif
+
+#if defined (__DJGPP__)
+# include <sys/farptr.h>
+# include <go32.h>
+# include <bios.h>
+#endif
+
+#ifndef _NKEYBRD_READ
+# define _NKEYBRD_READ		0x0
+#endif
+#ifndef _NKEYBRD_READY
+# define _NKEYBRD_READY		0x1
+#endif
+#ifndef _NKEYBRD_SHIFTSTATUS
+# define _NKEYBRD_SHIFTSTATUS	0x2
+#endif
 
 #define BIOSKEY		slbioskey
 #if defined(__WATCOMC__)
@@ -30,6 +47,12 @@
 # define keyWaiting()	BIOSKEY(_NKEYBRD_READY)
 #endif
 
+#ifndef __EMX__
+# define USE_MOUSE_CODE 1
+#else
+# define USE_MOUSE_CODE 0
+#endif
+
 #include "slang.h"
 #include "_slang.h"
 
@@ -188,6 +211,172 @@ static unsigned int slbioskey (int op)
 #endif
 }
 
+#if USE_MOUSE_CODE
+/*----------------------------------------------------------------------*\
+ * Simple mouse routines for 16/32-bit DOS-targets.
+ * Gisle Vanem <giva@bgnett.no>
+\*----------------------------------------------------------------------*/
+
+#define HARD_MOUSE_RESET 0
+
+static int Have_Mouse = 0;
+static int Process_Mouse_Events = 0;
+
+/*----------------------------------------------------------------------*\
+ * peem_far_mem()
+ *
+\*----------------------------------------------------------------------*/
+static unsigned long peek_dos_mem (unsigned long dos_addr,
+                                   unsigned char *pentry)
+{
+  unsigned long vector;
+  unsigned char entry;
+
+#if defined(__DJGPP__)
+# define MAKE_LINEAR(seg,ofs) ((unsigned long)(((seg) << 4) + (ofs)))
+  vector = _farpeekl (_dos_ds, dos_addr);
+  entry  = _farpeekb (_dos_ds, MAKE_LINEAR(vector >> 16, vector & 0xffff));
+
+#elif defined(__EMX__)
+  vector = 0;
+  entry  = 0;  /* to-do!! */
+
+#elif defined(__WATCOMC__) && defined(__FLAT__)  /* wcc386 */
+  vector = *(unsigned long*) dos_addr;
+  entry  = *(unsigned char*) vector;
+#else
+  vector = *(unsigned long far*) dos_addr;
+  entry  = *(unsigned char far*) vector;
+#endif
+
+   if (pentry)
+     *pentry = entry;
+   return (vector);
+}
+
+/*----------------------------------------------------------------------*\
+ * mouse_pressed (int button, int *x_pos, int *y_pos)
+ *
+ * Return 1 if left (0) or right (1) mouse-button was pressed.
+\*----------------------------------------------------------------------*/
+static int mouse_pressed (int button, int *x_pos, int *y_pos)
+{
+   union REGS r;
+
+   r.x.ax = 5;
+   r.x.bx = button;
+   int86 (0x33, &r, &r);
+   *x_pos = r.x.cx;
+   *y_pos = r.x.dx;
+   return (r.x.bx);
+}
+
+/*----------------------------------------------------------------------*\
+ * mouse_show (int show)
+ *
+ * Show (show=1) or hide (show=0) the mouse cursor
+\*----------------------------------------------------------------------*/
+static int mouse_show (int show)
+{
+   union REGS r;
+
+   if (Have_Mouse == 0)
+     return -1;
+
+   r.x.ax = show ? 1 : 2;
+   int86 (0x33, &r, &r);
+   return 0;
+}
+
+/*----------------------------------------------------------------------*\
+ * mouse_exit (void)
+ *
+ * Do a soft-reset of the mouse-driver (hides cursor)
+\*----------------------------------------------------------------------*/
+static void mouse_exit (void)
+{
+   union REGS r;
+   r.x.ax = 0x21;
+   int86 (0x33, &r, &r);
+}
+
+/*----------------------------------------------------------------------*\
+ * mouse_init (void)
+ *
+ * Peek at mouse interrupt vector for a driver.
+ * Do a soft/hard-reset of the mouse-driver.
+ * Add a SLang atexit function
+\*----------------------------------------------------------------------*/
+static int mouse_init (void)
+{
+   union REGS r;
+   unsigned char entry  = 0;
+   unsigned long vector = peek_dos_mem (4*0x33, &entry);
+
+   if (!vector || entry == 0xCF) /* NULL or points to IRET */
+     return -1;
+
+#if HARD_MOUSE_RESET
+   r.x.ax = 0;              /* mouse hard-reset and reinit */
+#else
+   r.x.ax = 0x21;           /* mouse soft-reset and reinit */
+#endif
+
+   int86 (0x33, &r, &r);
+   if (r.x.ax != 0xFFFF)
+     return -1;
+
+   (void) SLang_add_cleanup_function (mouse_exit);
+   Have_Mouse = 1;
+   return 0;
+}
+
+/*----------------------------------------------------------------------*\
+ * static int mouse_get_event (void);
+ *
+ * Poll mouse for changed button-state and encode x/y position and
+ * button state into an escape sequence "\e[M.."
+\*----------------------------------------------------------------------*/
+static int mouse_get_event (void)
+{
+   char buf [6];
+   int  x, y;
+
+   if (!Have_Mouse || Process_Mouse_Events == 0)
+      return (0);
+
+   if (mouse_pressed(0, &x, &y))       /* left button pressed? */
+     buf[3] = 040;
+   else if (mouse_pressed(1, &x, &y))  /* right button pressed */
+     buf[3] = 041;
+   else return (0);
+
+#if 0  /* test */
+   fprintf (stderr, "mouse_get_event: x=%d, y=%d\n", x, y);
+#endif
+
+   /*
+    * Taken from slw32tty.c / process_mouse_event():
+    *
+    * We have a simple press or release.  Encode it as an escape sequence
+    * and buffer the result.  The encoding is:
+    *   'ESC [ M b x y'
+    *  where b represents the button state, and x,y represent the coordinates.
+    * The ESC is handled by the calling routine.
+    */
+   buf[0] = 27;
+   buf[1] = '[';
+   buf[2] = 'M';
+   buf[4] = 1 + ' ' + (x >> 3); /* textmode co-ordinates are 1/8th of */
+   buf[5] = 1 + ' ' + (y >> 3); /* graphics-mode co-ordinates */
+
+   if (SLang_buffer_keystring (buf, sizeof(buf)) < 0)
+     return (0);
+   return (1);
+}
+
+#endif				       /* USE_MOUSE_CODE */
+
 /*----------------------------------------------------------------------*\
  *  Function:	int SLang_init_tty (int abort_char, int no_flow_control,
  *				    int opost);
@@ -210,8 +399,6 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 #endif
 
    set_ctrl_break (0);
-   /* clear keyboard buffer */
-   /* while (keyWaiting()) BIOSKEY(_NKEYBRD_READ); */
 
    return 0;
 }
@@ -243,9 +430,13 @@ int _SLsys_input_pending (int tsecs)
    if (tsecs < 0)
      tsecs = -tsecs / 100;
 
-   while (tsecs > 0)
+   while ((tsecs > 0) && (SLang_Input_Buffer_Len == 0))
      {
 	delay (20);	/* 20 ms or 1/50 sec */
+#if USE_MOUSE_CODE
+	if (1 == mouse_get_event ())
+	  return SLang_Input_Buffer_Len;
+#endif
 	if (keyWaiting()) break;
 	tsecs--;
      }
@@ -255,7 +446,7 @@ int _SLsys_input_pending (int tsecs)
 /*----------------------------------------------------------------------*\
  *  Function:	unsigned int _SLsys_getkey (void);
  *
- * wait for and get the next available keystroke.
+ * Wait for and get the next available mouse-event / keystroke.
  * Also re-maps some useful keystrokes.
  *
  *	Backspace (^H)	=>	Del (127)
@@ -266,14 +457,12 @@ unsigned int _SLsys_getkey (void)
 {
    unsigned int key, scan, ch, shift;
 
-#if 0
-   int tsecs = SL_input_time;	/* tsecs = 300 */
-   if ((SLgetkey_hook != NULL) && !keyWaiting())
-     {
-	while (!_SLsys_input_pending(tsecs))
-	  getkey_hook ();
-     }
-#endif
+   while ((SLang_Input_Buffer_Len == 0)
+	  && (0 == _SLsys_input_pending (300)))
+     ;
+
+   if (SLang_Input_Buffer_Len)
+     return SLang_getkey ();
 
    key  = BIOSKEY(_NKEYBRD_READ);
    ch   = key & 0xff;
@@ -309,9 +498,22 @@ int SLang_set_abort_signal (void (*handler)(int))
 
 int SLtt_set_mouse_mode (int mode, int force)
 {
-   /* FIXME: Priority=low */
+#if USE_MOUSE_CODE
+   (void) force;
+
+   if ((Have_Mouse == 0)
+       && (-1 == mouse_init ()))
+   {
+      Process_Mouse_Events = 0;
+      return -1;
+   }
+
+   Process_Mouse_Events = mode;
+   return mouse_show (mode);
+#else
    (void) mode;
    (void) force;
 
    return -1;
+#endif
 }
