@@ -1,5 +1,5 @@
 /* error handling common to all routines. */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -51,7 +51,9 @@ static char *get_error_string (void)
       case SL_USAGE_ERROR:
 	str = "Illegal usage of function";
 	break;
-
+      case SL_FLOATING_EXCEPTION:
+	str = "Floating Point Exception";
+	break;
       case SL_UNKNOWN_ERROR:
       default: str = "Unknown Error Code";
      }
