@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999 John E. Davis
+/* Copyright (c) 1998, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -48,25 +48,27 @@ int SLkp_init (void)
    /* Note: This will not work if SLgetkey_map_to_ansi (1) has
     * been called.
     */
-    SLkm_define_keysym ("^@\x48", SL_KEY_UP, Keymap_List );
-    SLkm_define_keysym ("^@\x50", SL_KEY_DOWN, Keymap_List );
-    SLkm_define_keysym ("^@\x4d", SL_KEY_RIGHT, Keymap_List );
-    SLkm_define_keysym ("^@\x4b", SL_KEY_LEFT, Keymap_List );
-    SLkm_define_keysym ("^@\x47", SL_KEY_HOME, Keymap_List );
-    SLkm_define_keysym ("^@\x49", SL_KEY_PPAGE, Keymap_List );
-    SLkm_define_keysym ("^@\x51", SL_KEY_NPAGE, Keymap_List );
-    SLkm_define_keysym ("^@\x4f", SL_KEY_END, Keymap_List );
-    SLkm_define_keysym ("^@\x52", SL_KEY_IC, Keymap_List );
-
-    SLkm_define_keysym ("\xE0\x48", SL_KEY_UP, Keymap_List );
-    SLkm_define_keysym ("\xE0\x50", SL_KEY_DOWN, Keymap_List );
-    SLkm_define_keysym ("\xE0\x4d", SL_KEY_RIGHT, Keymap_List );
-    SLkm_define_keysym ("\xE0\x4b", SL_KEY_LEFT, Keymap_List );
-    SLkm_define_keysym ("\xE0\x47", SL_KEY_HOME, Keymap_List );
-    SLkm_define_keysym ("\xE0\x49", SL_KEY_PPAGE, Keymap_List );
-    SLkm_define_keysym ("\xE0\x51", SL_KEY_NPAGE, Keymap_List );
-    SLkm_define_keysym ("\xE0\x4f", SL_KEY_END, Keymap_List );
-    SLkm_define_keysym ("\xE0\x52", SL_KEY_IC, Keymap_List );
+   SLkm_define_keysym ("^@\x48", SL_KEY_UP, Keymap_List );
+   SLkm_define_keysym ("^@\x50", SL_KEY_DOWN, Keymap_List );
+   SLkm_define_keysym ("^@\x4d", SL_KEY_RIGHT, Keymap_List );
+   SLkm_define_keysym ("^@\x4b", SL_KEY_LEFT, Keymap_List );
+   SLkm_define_keysym ("^@\x47", SL_KEY_HOME, Keymap_List );
+   SLkm_define_keysym ("^@\x49", SL_KEY_PPAGE, Keymap_List );
+   SLkm_define_keysym ("^@\x51", SL_KEY_NPAGE, Keymap_List );
+   SLkm_define_keysym ("^@\x4f", SL_KEY_END, Keymap_List );
+   SLkm_define_keysym ("^@\x52", SL_KEY_IC, Keymap_List );
+   SLkm_define_keysym ("^@\x53", SL_KEY_DELETE, Keymap_List );
+   
+   SLkm_define_keysym ("\xE0\x48", SL_KEY_UP, Keymap_List );
+   SLkm_define_keysym ("\xE0\x50", SL_KEY_DOWN, Keymap_List );
+   SLkm_define_keysym ("\xE0\x4d", SL_KEY_RIGHT, Keymap_List );
+   SLkm_define_keysym ("\xE0\x4b", SL_KEY_LEFT, Keymap_List );
+   SLkm_define_keysym ("\xE0\x47", SL_KEY_HOME, Keymap_List );
+   SLkm_define_keysym ("\xE0\x49", SL_KEY_PPAGE, Keymap_List );
+   SLkm_define_keysym ("\xE0\x51", SL_KEY_NPAGE, Keymap_List );
+   SLkm_define_keysym ("\xE0\x4f", SL_KEY_END, Keymap_List );
+   SLkm_define_keysym ("\xE0\x52", SL_KEY_IC, Keymap_List );
+   SLkm_define_keysym ("\xE0\x53", SL_KEY_DELETE, Keymap_List );
 
     strcpy (esc_seq, "^@ ");	       /* guarantees esc_seq[3] = 0. */
 
