@@ -1,5 +1,5 @@
 /* sin, cos, etc, for S-Lang */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -27,7 +27,7 @@
 static void math_floating_point_exception (int sig)
 {
    sig = errno;
-   SLang_Error = SL_INTRINSIC_ERROR;
+   if (SLang_Error == 0) SLang_Error = SL_FLOATING_EXCEPTION;
    (void) SIGNAL (SIGFPE, math_floating_point_exception);
    errno = sig;
 }
