@@ -1,24 +1,23 @@
-/* Copyright (c) 1992, 1995 John E. Davis
- * All rights reserved.
- * 
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
+ * This file is part of the S-Lang library.
+ *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
 
-
 /* These routines are fast memcpy, memset routines.  When available, I
    use system rouines.  For msdos, I use inline assembly. */
 
 /* The current versions only work in the forward direction only!! */
-      
-#include <stdio.h>
-#include "config.h"
+
+#include "slinclud.h"
+
 #include "slang.h"
 #include "_slang.h"
 
-char *jed_memcpy(char *s1, char *s2, int n)
+char *SLmemcpy(char *s1, char *s2, int n)
 {
-#if defined(msdos) && !defined(__WIN32__)
+#if defined(__BORLANDC__) && defined(__MSDOS__)
    asm mov ax, ds
    asm mov bx, si
    asm mov dx, di
@@ -31,20 +30,20 @@ char *jed_memcpy(char *s1, char *s2, int n)
    asm mov si, bx
    asm mov di, dx
    return(s1);
-   
+
 #else
    register char *smax, *s = s1;
    int n2;
-   
+
    n2 = n % 4;
    smax = s + (n - 4);
-   while (s <= smax) 
+   while (s <= smax)
      {
 	*s = *s2; *(s + 1) = *(s2 + 1); *(s + 2) = *(s2 + 2); *(s + 3) = *(s2 + 3);
 	s += 4;
 	s2 += 4;
      }
    while (n2--) *s++ = *s2++;
-   return(s1); 
+   return(s1);
 #endif
 }
