@@ -1,5 +1,5 @@
 /* file intrinsics for S-Lang */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -12,7 +12,7 @@
 # include <sys/types.h>
 #endif
 
-#if defined (__EMX__) || defined(__BORLANDC__) || defined(__IBMC__) || defined(_MSC_VER)
+#ifdef HAVE_IO_H
 # include <io.h>		       /* for chmod */
 #endif
 
@@ -400,8 +400,100 @@ static int chdir_cmd (char *s)
    return ret;
 }
 
+#ifdef VMS
+static int remove_cmd (char *);
+/* If the file looks like xxx, then change it to xxx.dir.  If
+ * it looks like A:[B.xxx] then change it to A:[B]xxx.dir.
+ */
+
+static char *vms_convert_dirspec_to_vms_dir (char *str)
+{
+   char *s;
+   char *version;
+   unsigned int len;
+   char *dot;
+
+   len = strlen (str);
+
+   version = strchr (str, ';');
+   if (version == NULL)
+     version = str + len;
+   /* version points to the version of the input string */
+
+   
+   if (NULL == (s = SLmalloc (len + 8)))/* allow extra space to work with */
+     return NULL;
+
+   len = (unsigned int) (version - str);
+   strncpy (s, str, len);
+   s[len] = 0;
+   str = s;
+   
+   /* Lowercase the whole thing */
+   while (*s != 0)
+     {
+	*s = LOWER_CASE(*s);
+	s++;
+     }
+
+   if ((s > str)
+       && (s[-1] != ']'))
+     {
+	if ((s >= str + 4)
+	    && (0 == strcmp (s - 4, ".dir")))
+	  s -= 4;
+	goto add_dir_version;
+     }
+
+   /* Check for one of two possibilities:
+    * 
+    *     dev:[x]   --> dev:x
+    *     dev:[a.x] --> dev:[a]x
+    */
+   
+   if (NULL == (dot = strchr (str, '.')))
+     {
+	/* First possibility */
+	if (NULL == (s = strchr (str, '[')))
+	  return str;		       /* let someone else figure this out */
+	while (s[1] != ']')
+	  {
+	     s[0] = s[1];
+	     s++;
+	  }
+	*s = 0;
+	goto add_dir_version;
+     }
+   
+   while (NULL != (s = strchr (dot + 1, '.')))
+     dot = s;
+   
+   *dot = ']';
+   s = str + (len - 1);
+   
+   /* Drop */
+
+   add_dir_version:
+   strcpy (s, ".dir");
+   strcpy (s+4, version);
+   return str;
+}
+#endif
+
 static int rmdir_cmd (char *s)
 {
+#ifdef VMS
+   int status;
+
+   if (NULL == (s = vms_convert_dirspec_to_vms_dir (s)))
+     return -1;
+   
+   status = remove_cmd (s);
+   SLfree (s);
+   
+   return status;
+
+#else
    int ret;
 
    while (-1 == (ret = rmdir (s)))
@@ -414,6 +506,7 @@ static int rmdir_cmd (char *s)
 	break;
      }
    return ret;
+#endif
 }
 
 static int remove_cmd (char *s)
