@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -28,6 +28,9 @@ static unsigned char *search_forward (register unsigned char *beg,
 
    str_len = (int) (end - beg);
    if (str_len < key_len) return (NULL);
+   
+   if (key_len == 0)
+     return NULL;
 
    char1 = key[key_len - 1];
    beg += (key_len - 1);
@@ -73,7 +76,10 @@ static unsigned char *search_backward (unsigned char *beg,unsigned char *end,
 
     str_len = (int) (end - beg);
     if (str_len < key_len) return (NULL);
-
+   
+   if (key_len == 0)
+     return NULL;
+	
    /*  end -= (key_len - 1); */
    end -= key_len;
 
