@@ -1,5 +1,5 @@
 /* time related system calls */
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -21,14 +21,45 @@
 
 #include <errno.h>
 
-/* Sleep is defined badly in MSVC... */
-#ifdef _MSC_VER
-# define sleep(n) _sleep((n)*1000)
-#endif
-
 #include "slang.h"
 #include "_slang.h"
 
+#ifdef __WIN32__
+#include <windows.h>
+/* Sleep is defined badly in MSVC... */
+# ifdef _MSC_VER
+#  define sleep(n) _sleep((n)*1000)
+# else
+#  ifdef sleep
+#   undef sleep
+#  endif
+#  define sleep(x) if(x)Sleep((x)*1000)
+# endif
+#endif
+
+
+#if defined(IBMPC_SYSTEM)
+/* For other system (Unix and VMS), _SLusleep is in sldisply.c */
+int _SLusleep (unsigned long s)
+{
+   sleep (s/1000000L);
+   s = s % 1000000L;
+
+# if defined(__WIN32__)
+   Sleep (s/1000);
+#else
+# if defined(__IBMC__)
+   DosSleep(s/1000);
+# else
+#  if defined(_MSC_VER)
+   _sleep (s/1000);
+#  endif
+# endif
+#endif
+   return 0;
+}
+#endif
+
 #if defined(__IBMC__) && !defined(_AIX)
 /* sleep is not a standard function in VA3. */
 unsigned int sleep (unsigned int seconds)
@@ -47,10 +78,28 @@ static char *ctime_cmd (unsigned long *tt)
    return (t);
 }
 
-static void sleep_cmd (int *secs)
+static void sleep_cmd (void)
 {
-   if (*secs > 0)
-     sleep (*secs);
+   unsigned int secs;
+#if SLANG_HAS_FLOAT
+   unsigned long usecs;
+   double x;
+
+   if (-1 == SLang_pop_double (&x, NULL, NULL))
+     return;
+
+   if (x < 0.0) 
+     x = 0.0;
+   secs = (unsigned int) x;
+   sleep (secs);
+   x -= (double) secs;
+   usecs = (unsigned long) (1e6 * x);
+   if (usecs > 0) _SLusleep (usecs);
+#else
+   if (-1 == SLang_pop_uinteger (&secs))
+     return;
+   if (secs != 0) sleep (secs);
+#endif
 }
 
 static unsigned long _time_cmd (void)
@@ -237,7 +286,7 @@ static double toc_cmd (void)
 static SLang_Intrin_Fun_Type Time_Funs_Table [] =
 {
    MAKE_INTRINSIC_1("ctime", ctime_cmd, SLANG_STRING_TYPE, SLANG_ULONG_TYPE),
-   MAKE_INTRINSIC_I("sleep", sleep_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("sleep", sleep_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_time", _time_cmd, SLANG_ULONG_TYPE),
    MAKE_INTRINSIC_0("time", SLcurrent_time_string, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_1("localtime", localtime_cmd, SLANG_VOID_TYPE, SLANG_LONG_TYPE),
