@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999 John E. Davis
+/* Copyright (c) 1998, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -1189,7 +1189,7 @@ int SLang_guess_type (char *t)
 #endif				       /* SLANG_HAS_FLOAT */
 }
 
-static unsigned long hex_atoul (unsigned char *s)
+static int hex_atoul (unsigned char *s, unsigned long *ul)
 {
    register unsigned char ch;
    register unsigned long value;
@@ -1223,7 +1223,8 @@ static unsigned long hex_atoul (unsigned char *s)
 	   case 'u':
 	   case 'l':
 	   case 'h':
-	     return (long) value;
+	     *ul = value;
+	     return 0;
 
 	   case '8':
 	   case '9':
@@ -1252,7 +1253,8 @@ static unsigned long hex_atoul (unsigned char *s)
 	  }
 	value = value * base + ch1;
      }
-   return value;
+   *ul = value;
+   return 0;
 }
 
 /* Note: These routines do not check integer overflow.  I would use the C
@@ -1273,7 +1275,10 @@ unsigned long SLatoul (unsigned char *s)
      }
 
    if (*s == '0')
-     value = hex_atoul(s);
+     {
+	if (-1 == hex_atoul (s, &value))
+	  return (unsigned long) -1;
+     }
    else
      {
 	while (WHITE_CHAR == CHAR_CLASS(*s))
@@ -1470,7 +1475,7 @@ static int escape_string (unsigned char *s, unsigned char *smax,
 	     if (buf < buf_max) *buf++ = '0';
 	     if (buf < buf_max) *buf++ = '0';
 	     *is_escaped = 1;
-	     return 0;
+	     break; /* return 0; */
 
 	   case '\n':
 	     *buf++ = '\\';
