@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold -*- */
-/* Copyright (c) 1992, 1997 John E. Davis
+/* Copyright (c) 1992, 1997, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -28,6 +28,9 @@ int SLtt_Msdos_Cheap_Video = 0;
 
 void (*_SLtt_color_changed_hook)(void);
 
+/* This definition will need changing when SLsmg_Char_Type changes. */
+#define SLSMG_CHAR_TO_USHORT(x) ((unsigned short)(x))
+
 /*{{{ ------------- static local variables ---------- */
 
 static int Attribute_Byte;
@@ -417,7 +420,7 @@ static unsigned short *asm_video_write (register unsigned char *pp,
      return (pos + n);
 }
 
-static void write_attributes (unsigned short *src, unsigned int count)
+static void write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    register unsigned char *p;
    register unsigned short pair;
@@ -429,7 +432,8 @@ static void write_attributes (unsigned short *src, unsigned int count)
 
    while (count--)
      {
-	pair = *(src++);	/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);	/* character/color pair */
+	src++;
 	ch = pair & 0xff;	/* character value */
 	color = pair >> 8;	/* color value */
 	if (color != Current_Color)	/* need a new color */
@@ -797,7 +801,7 @@ void SLtt_del_eol (void)
 }
 
 static void
-write_attributes (unsigned short *src, unsigned int count)
+write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    register unsigned short pair;
    unsigned int n;
@@ -806,7 +810,8 @@ write_attributes (unsigned short *src, unsigned int count)
    n = Cursor_Col;
    while (count)
      {
-	pair   = *(src++);		/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);/* character/color pair */
+	src++;
 	SLtt_reverse_video (pair >> 8);	/* color change */
 	ScreenPutChar ((int)pair & 0xFF, Attribute_Byte, n, Cursor_Row);
 	n++;
@@ -1036,7 +1041,7 @@ void SLtt_del_eol (void)
 }
 
 static void
-write_attributes (unsigned short *src, unsigned int count)
+write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    register unsigned char *p = Line_Buffer;
    register unsigned short pair;
@@ -1045,7 +1050,8 @@ write_attributes (unsigned short *src, unsigned int count)
    /* write into a character/attribute pair */
    while (n-- > 0)
      {
-	pair   = *(src++);		/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);/* character/color pair */
+	src++;
 	SLtt_reverse_video (pair >> 8);	/* color change */
 	*(p++) = pair & 0xff;		/* character byte */
 	*(p++) = Attribute_Byte;	/* attribute byte */
@@ -1357,7 +1363,7 @@ void SLtt_del_eol (void)
 }
 
 static void
-write_attributes (unsigned short *src, unsigned int count)
+write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    unsigned short pair;
    COORD coord, c;
@@ -1371,7 +1377,8 @@ write_attributes (unsigned short *src, unsigned int count)
    while (n)
      {
 	n--;
-	pair   = *(src++);		/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);/* character/color pair */
+	src++;
 	SLtt_reverse_video (pair >> 8);	/* color change */
 	p->Char.AsciiChar = pair & 0xff;
 	p->Attributes = Attribute_Byte;
@@ -1456,9 +1463,13 @@ void SLtt_get_screen_size (void)
 	SLang_exit_error ("Unable to determine the screen size");
 	return;
      }
-
+#if 0
    SLtt_Screen_Rows = csbi.dwSize.Y;
    SLtt_Screen_Cols = csbi.dwSize.X;
+#else
+   SLtt_Screen_Rows = (csbi.srWindow.Bottom - csbi.srWindow.Top) + 1;
+   SLtt_Screen_Cols = (csbi.srWindow.Right - csbi.srWindow.Left) + 1;
+#endif
 }
 
 void SLtt_get_terminfo (void)
@@ -1711,7 +1722,7 @@ void SLtt_del_eol (void)
 }
 
 static void
-write_attributes (unsigned short *src, unsigned int count)
+write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    register unsigned char *p = Line_Buffer;
    register unsigned short pair;
@@ -1720,7 +1731,8 @@ write_attributes (unsigned short *src, unsigned int count)
    /* write into a character/attribute pair */
    while (n-- > 0)
      {
-	pair   = *(src++);		/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);/* character/color pair */
+	src++;
 	SLtt_reverse_video (pair >> 8);	/* color change */
 	*(p++) = pair & 0xff;		/* character byte */
 	*(p++) = Attribute_Byte;	/* attribute byte */
@@ -2028,7 +2040,7 @@ void SLtt_del_eol (void)
 }
 
 static void
-write_attributes (unsigned short *src, unsigned int count)
+write_attributes (SLsmg_Char_Type *src, unsigned int count)
 {
    register unsigned short pair;
    register unsigned short *pos = MK_SCREEN_POINTER (Cursor_Row, 0);
@@ -2036,7 +2048,8 @@ write_attributes (unsigned short *src, unsigned int count)
    /* write into a character/attribute pair */
    while (count--)
      {
-	pair   = *(src++);		/* character/color pair */
+	pair = SLSMG_CHAR_TO_USHORT(*src);/* character/color pair */
+	src++;
 	SLtt_reverse_video (pair >> 8);	/* color change */
 	*(pos++) = ((unsigned short) Attribute_Byte << 8) | (pair & 0xff);
      }
@@ -2162,8 +2175,19 @@ int SLtt_flush_output (void)
 
 int SLtt_set_cursor_visibility (int show)
 {
+#if defined(WIN32_VIDEO)
+   CONSOLE_CURSOR_INFO c;
+     
+   if (0 == GetConsoleCursorInfo (hStdout, &c))
+     return -1;
+   c.bVisible = (show ? TRUE: FALSE);
+   if (0 == SetConsoleCursorInfo (hStdout, &c))
+     return -1;
+   return 0;
+#else
    (void) show;
    return -1;
+#endif
 }
 
 void SLtt_set_mono (int obj_unused, char *unused, SLtt_Char_Type c_unused)
@@ -2200,8 +2224,8 @@ void SLtt_normal_video (void)
 }
 
 /*----------------------------------------------------------------------*\
- * Function:	void SLtt_smart_puts (unsigned short *new_string,
- *				      unsigned short *old_string,
+ * Function:	void SLtt_smart_puts (SLsmg_Char_Type *new_string,
+ *				      SLsmg_Char_Type *old_string,
  *				      int len, int row);
  *
  * puts NEW_STRING, which has length LEN, at row ROW.  NEW_STRING contains
@@ -2211,8 +2235,8 @@ void SLtt_normal_video (void)
  *
  * OLD_STRING is not used, maintained for compatibility with other systems
 \*----------------------------------------------------------------------*/
-void SLtt_smart_puts (unsigned short *new_string,
-		      unsigned short *old_string,
+void SLtt_smart_puts (SLsmg_Char_Type *new_string,
+		      SLsmg_Char_Type *old_string,
 		      int len, int row)
 {
    (void) old_string;
