@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -66,6 +66,9 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 	_SLw32_Hstdin = INVALID_HANDLE_VALUE;
 	return -1;
      }
+   
+   if (abort_char > 0)
+     SLang_Abort_Char = abort_char;
 
    return 0;
 }
@@ -82,7 +85,7 @@ void SLang_reset_tty (void)
    set_ctrl_break (1);
 }
 
-static int process_mouse_event (MOUSE_EVENT_RECORD *m, int check_only)
+static int process_mouse_event (MOUSE_EVENT_RECORD *m)
 {
    char buf [8];
 
@@ -107,42 +110,160 @@ static int process_mouse_event (MOUSE_EVENT_RECORD *m, int check_only)
     *  where b represents the button state, and x,y represent the coordinates.
     * The ESC is handled by the calling routine.
     */
-   if (m->dwButtonState & 1) buf[2] = ' ';
-   else if (m->dwButtonState & 2) buf[2] = ' ' + 2;
-   else if (m->dwButtonState & 4) buf[2] = ' ' + 1;
+   if (m->dwButtonState & 1) buf[3] = ' ';
+   else if (m->dwButtonState & 2) buf[3] = ' ' + 2;
+   else if (m->dwButtonState & 4) buf[3] = ' ' + 1;
    else return -1;
 
-   if (check_only)
-     return 0;
+   buf[0] = 27;
+   buf[1] = '[';
+   buf[2] = 'M';
 
-   buf[3] = 1 + ' ' + m->dwMousePosition.X;
-   buf[4] = 1 + ' ' + m->dwMousePosition.Y;
 
-   buf[0] = '[';
-   buf[1] = 'M';
+   buf[4] = 1 + ' ' + m->dwMousePosition.X;
+   buf[5] = 1 + ' ' + m->dwMousePosition.Y;
 
-   return SLang_ungetkey_string (buf, 5);
+   return SLang_buffer_keystring (buf, 6);
 }
 
-/* Return TRUE if the Shift, Ctrl, Alt, Caps Lock, or Num Lock key was pressed.
- */
-static int is_shift_key (INPUT_RECORD *pRecord)
+static int process_key_event (KEY_EVENT_RECORD *key)
 {
-   unsigned int k;
-#if 0 /* defined(__MINGW32__) */
-   k = (pRecord->Event.KeyEvent.AsciiChar |
-	(pRecord->Event.KeyEvent.wVirtualScanCode << 8));
-#else
-   k = (pRecord->Event.KeyEvent.uChar.AsciiChar |
-	(pRecord->Event.KeyEvent.wVirtualScanCode << 8));
-#endif
+   unsigned int key_state = 0;
+   unsigned int scan;
+   char c1 = 0;
+   DWORD d = key->dwControlKeyState;
+   unsigned char buf[4];
 
-   return ((k == 0x1d00)
-	   || (k == 0x2a00)
-	   || (k == 0x3600)
-	   || (k == 0x3800)
-	   || (k == 0x3a00)
-	   || (k == 0x4500));
+   if (!key->bKeyDown) return 0;
+   if (d & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED))
+     key_state |= _SLTT_KEY_ALT;
+   if (d & (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED)) 
+     key_state |= _SLTT_KEY_CTRL;
+   if (d & SHIFT_PRESSED)
+     key_state |= _SLTT_KEY_SHIFT;
+
+   scan = key->wVirtualScanCode;
+
+   switch (scan)
+     {
+      case 0x00E:		       /* backspace */
+	return SLang_buffer_keystring ("\x7F", 1);
+
+      case 0x003:		       /* 2 key */
+	if (key_state & _SLTT_KEY_ALT)
+	  break;
+	/* Drop */
+      case 0x039: 		       /* space */
+	if (key_state & _SLTT_KEY_CTRL)
+	  return SLang_buffer_keystring ("\x00\x03", 2);
+	break;
+
+      case 0x007:		       /* 6 key */
+	if (_SLTT_KEY_CTRL == (key_state & (_SLTT_KEY_ALT|_SLTT_KEY_CTRL)))
+	  return SLang_buffer_keystring ("\x1E", 1);   /* Ctrl-^ */
+	break;
+
+      case 0x00C:		       /* -/_ key */
+	if (_SLTT_KEY_CTRL == (key_state & (_SLTT_KEY_ALT|_SLTT_KEY_CTRL)))
+	  return SLang_buffer_keystring ("\x1F", 1);
+	break;
+
+      case 0x00F:		       /* TAB */
+	if (_SLTT_KEY_SHIFT == key_state)
+	  return SLang_buffer_keystring ("\x00\x09", 2);
+	break;
+
+      case 0xE02F:		       /* KEYPAD SLASH */
+      case 0x037:		       /* KEYPAD STAR */
+      case 0x04A:		       /* KEYPAD MINUS */
+      case 0x04E:		       /* KEYPAD PLUS */
+	if (d & NUMLOCK_ON)
+	  break;
+      case 0x047:		       /* KEYPAD HOME */
+      case 0x048:		       /* KEYPAD UP */
+      case 0x049:		       /* KEYPAD PGUP */
+      case 0x04B:		       /* KEYPAD LEFT */
+      case 0x04C:		       /* KEYPAD 5 */
+      case 0x04D:		       /* KEYPAD RIGHT */
+      case 0x04F:		       /* KEYPAD END */
+      case 0x050:		       /* KEYPAD DOWN */
+      case 0x051:		       /* KEYPAD PGDN */
+      case 0x052:		       /* KEYPAD INSERT */
+      case 0x053:		       /* KEYPAD DEL */
+	if (d & ENHANCED_KEY)
+	  scan |= 0xE000;
+	else
+	  {
+	     if (d & NUMLOCK_ON)
+	       break;
+	  }
+	(void) _SLpc_convert_scancode (scan, key_state, 0);
+	return 0;
+
+      case 0x3b:		       /* F1 */
+      case 0x3c:
+      case 0x3d:
+      case 0x3e:
+      case 0x3f:
+      case 0x40:
+      case 0x41:
+      case 0x42:
+      case 0x43:
+      case 0x44:
+      case 0x57:
+      case 0x58:		       /* F12 */
+	(void) _SLpc_convert_scancode (scan, key_state, 0);
+     }
+   
+   c1 = key->uChar.AsciiChar;
+   if (c1 != 0)
+     {
+	if (_SLTT_KEY_ALT == (key_state & (_SLTT_KEY_ALT|_SLTT_KEY_CTRL)))
+	  {
+	     buf[0] = 27;
+	     buf[1] = c1;
+	     return SLang_buffer_keystring (buf, 2);
+	  }
+	if (c1 == SLang_Abort_Char)
+	  {
+	     if (SLang_Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
+	     SLKeyBoard_Quit = 1;
+	  }
+	buf[0] = c1;
+	return SLang_buffer_keystring (buf, 1);
+     }
+   return 0;
+}
+
+
+static void process_console_records(void)
+{
+   INPUT_RECORD record;
+   DWORD bytesRead;
+   DWORD n = 0;
+
+   if (FALSE == GetNumberOfConsoleInputEvents(_SLw32_Hstdin, &n))
+     return;
+
+   while (n > 0)
+     {
+	ReadConsoleInput(_SLw32_Hstdin, &record, 1, &bytesRead);
+	switch (record.EventType)
+	  {
+	   case KEY_EVENT:
+	     (void) process_key_event(&record.Event.KeyEvent);
+	     break;
+
+	   case MOUSE_EVENT:
+	     process_mouse_event(&record.Event.MouseEvent);
+	     break;
+
+	   case WINDOW_BUFFER_SIZE_EVENT:
+	     /* process_resize_records(&record.Event.WindowBufferSizeEvent); */
+	     break;
+	  }
+	n--;
+     }
 }
 
 /*----------------------------------------------------------------------*\
@@ -152,9 +273,7 @@ static int is_shift_key (INPUT_RECORD *pRecord)
 \*----------------------------------------------------------------------*/
 int _SLsys_input_pending (int tsecs)
 {
-   INPUT_RECORD record;
-   DWORD events_read;
-   DWORD ms;
+   long ms;
 
    if (_SLw32_Hstdin == INVALID_HANDLE_VALUE)
      return -1;
@@ -162,39 +281,20 @@ int _SLsys_input_pending (int tsecs)
    if (tsecs < 0) ms = -tsecs;	       /* specifies 1/1000 */
    else ms = tsecs * 100L;	       /* convert 1/10 to 1/1000 secs */
 
-   if (ms)
-     (void) WaitForSingleObject (_SLw32_Hstdin, ms);
-
-   while (1)
+   process_console_records ();
+   while ((ms > 0)
+	  && (SLang_Input_Buffer_Len == 0))
      {
-	if (0 == PeekConsoleInput(_SLw32_Hstdin, &record, 1, &events_read))
-	  /* function failed */
-	  return 0;
+	long t;
 
-	if (events_read != 1)
-	  return 0;
+	t = GetTickCount ();
 
-	switch (record.EventType)
-	  {
-	   default:
-	     break;
-
-	   case MOUSE_EVENT:
-	     if (0 == process_mouse_event (&record.Event.MouseEvent, 1))
-	       return 1;
-
-	     break;
-
-	   case KEY_EVENT:
-	     if (record.Event.KeyEvent.bKeyDown
-		 && (0 == is_shift_key (&record)))
-	       return 1;
-	     break;
-	  }
-
-	/* something else is here, so read it and try again */
-	(void) ReadConsoleInput(_SLw32_Hstdin, &record, 1, &events_read);
+	(void) WaitForSingleObject (_SLw32_Hstdin, ms);
+	process_console_records ();
+	ms -= GetTickCount () - t;
      }
+   
+   return SLang_Input_Buffer_Len;
 }
 
 /*----------------------------------------------------------------------*\
@@ -209,55 +309,28 @@ int _SLsys_input_pending (int tsecs)
 \*----------------------------------------------------------------------*/
 unsigned int _SLsys_getkey (void)
 {
-   unsigned int ch;
-   DWORD events_read;
-   INPUT_RECORD record;
+   /* Check the input buffer because _SLsys_input_pending may have been 
+    * called prior to this to stuff the input buffer.
+    */
+   if (SLang_Input_Buffer_Len)
+     return SLang_getkey ();
 
    if (_SLw32_Hstdin == INVALID_HANDLE_VALUE)
      return SLANG_GETKEY_ERROR;
 
    while (1)
      {
-	if (!ReadConsoleInput(_SLw32_Hstdin, &record, 1, &events_read))
-	  return 0;
+	int status;
 
-	switch (record.EventType)
-	  {
-	   default:
-	     break;
-
-	   case MOUSE_EVENT:
-	     if (0 == process_mouse_event (&record.Event.MouseEvent, 0))
-	       return 27;	       /* ESC */
-	     break;
-
-	   case KEY_EVENT:
-	     if ((0 == record.Event.KeyEvent.bKeyDown)
-		 || is_shift_key (&record))
-	       break;
-
-#if 0 /* defined(__MINGW32__) */
-	     ch = record.Event.KeyEvent.AsciiChar;
-#else
-	     ch = record.Event.KeyEvent.uChar.AsciiChar;
-#endif
-	     switch (ch)
-	       {
-		case 8:		       /* ^H --> 127 */
-		  if (record.Event.KeyEvent.wVirtualScanCode == 0x0E)
-		    ch = 127;
-		  break;
-
-		case 0:
-		case 0xE0:
-		  ch = _SLpc_convert_scancode (record.Event.KeyEvent.wVirtualScanCode);
-		  break;
-
-		default:
-		  break;
-	       }
-	     return ch;
-	  }
+	if (SLKeyBoard_Quit)
+	  return SLang_Abort_Char;
+	
+	status = _SLsys_input_pending (600);
+	if (status == -1)
+	  return SLANG_GETKEY_ERROR;
+	
+	if (status > 0)
+	  return SLang_getkey ();
      }
 }
 
